import os
import select
import socket
import sys


def SocketError(Exception):
    def __init__(self):
        super(SocketError, self).__init__("Socket error")


def recv_some(sock):
    try:
        msg = sock.recv(4096)
    except socket.error:
        raise SocketError()


class ManInTheMiddle(object):

    def __init__(self, opts, ctrl_file = sys.stdin):
        self.server_addr_port = (opts.server_addr, opts.server_port)
        self.ctrl_file = ctrl_file
        self.connections = []
        self.listen_addr = opts.listen_addr
        self.listen_port = opts.listen_port
        self.set_up_server()

    def set_up_server(self):
        self.server = socket.socket(socket.AF_INET, socket.SOCK_STREAM, 0)
        self.server.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.server.bind((self.listen_addr, self.listen_port))
        self.server.listen(10)

    def close_server(self):
        self.server.close()
        self.server = None

    def set_up_connection(self):
        client_side, client_addr = self.server.accept()
        server_side = socket.socket(socket.AF_INET, socket.SOCK_STREAM, 0)
        server_side.connect(self.server_addr_port)
        self.connections.append((client_side, server_side, client_addr))

    def process_connections(self, ready):
        i = 0
        while i < len(self.connections):
            cln, srv, client_addr = self.connections[i]
            must_tear_down = False
            try:
                if cln in ready:
                    cln_status = self.handle_transfer(client_addr, cln, srv, True)
                    must_tear_down = must_tear_down or cln_status
                if srv in ready:
                    srv_status = self.handle_transfer(client_addr, srv, cln, False)
                    must_tear_down = must_tear_down or srv_status
            except SocketError:
                must_tear_down = True
            if must_tear_down:
                cln.close()
                srv.close()
                del self.connections[i]
            else:
                i += 1

    def send_all(self, sock, msg):
        try:
            while len(msg) > 0:
                n = sock.send(msg)
                if n == 0:
                    raise SocketError()
                msg = msg[n:]
        except socket.error:
            raise SocketError()
        except IOError:
            raise SocketError()
        except OSError:
            raise SocketError()

    def handle_transfer(self, addr, src, dest, client_to_server):
        data = src.recv(4096)
        if len(data) == 0:
            if client_to_server:
                print "Client {} closing connection to server.".format(addr)
            else:
                print "Server closing connection to client {}.".format(addr)
            return True
        else:
            if client_to_server:
                print "Client {} to server: {} [{} bytes]".format(addr, repr(data[0:30]), len(data))
            else:
                print "Server to client {}: {} [{} bytes]".format(addr, repr(data[0:30]), len(data))
            self.send_all(dest, data)
            return False

    def handle_ctrl(self):
        return False

    def serve(self):
        while True:
            ready, _, _ = select.select(
                    (
                        ([] if self.server is None else [self.server]) +
                        [self.ctrl_file] +
                        sum(([cln, srv] for cln, srv, _ in self.connections), [])
                        ),
                    [],
                    [],
                    1.0
                    )
            if self.ctrl_file in ready:
                if not self.handle_ctrl():
                    break
            if self.server in ready:
                self.set_up_connection()
            self.process_connections(ready)

    def take_down(self):
        for cln, srv in self.connections:
            cln.close()
            srv.close()
        self.close_server()

    def ctrl_file_read(self, n):
        return os.read(self.ctrl_file.fileno(), n)

    def ctrl_file_write(self, s):
        return os.write(self.ctrl_file.fileno(), s)


def run(mitm_klass, arg_parser, ctrl_file = sys.stdin):
    arg_parser.add_argument(
            "listen_addr",
            help = "Address to listen on with the middle man."
            )
    arg_parser.add_argument(
            "listen_port",
            type = int,
            help = "Port to listen on with the middle man."
            )
    arg_parser.add_argument(
            "server_addr",
            help = "Address of the server."
            )
    arg_parser.add_argument(
            "server_port",
            type = int,
            help = "Port the server is listening on."
            )
    ns = arg_parser.parse_args(sys.argv[1:])

    mitm = mitm_klass(ns, ctrl_file = ctrl_file)
    print "Ready!"
    mitm.serve()
    print "Going down..."
    mitm.take_down()


# EOF
