import argparse
from connection import Client
import atexit
import msgs
import os
import sys


parser = argparse.ArgumentParser(
    description = "Uploads and concatenates files from the originator host of this program's caller, writing the contents to standard output."
    )
parser.add_argument(
    "path",
    nargs = "+",
    help = "Paths to a file to upload."
    )
ns = parser.parse_args(sys.argv[1:])

missing = [v for v in ["RESHD_SERVICE", "RESHD_SESSION"] if v not in os.environ]
if any(missing):
  print >> sys.stderr, "Missing environ variables:", ", ".join(missing)
  sys.exit(-1)

conn = Client(eval(os.environ["RESHD_SERVICE"]))
atexit.register(conn.close)
try:
  msgs.send(
      conn,
      code = "lcat",
      session = int(os.environ["RESHD_SESSION"]),
      paths = ns.path
      )
  resp = msgs.recv(conn)
  if "error" in resp:
    if resp["error"] == "session":
      print >> sys.stderr, "Unknown session ID:", os.environ["RESHD_SESSION"]
    elif resp["error"] == "unreadable":
      print >> sys.stderr, "These files cannot be read on the host:", ", ".join(resp["paths"])
    sys.exit(-2)
except msgs.Error:
  print >> sys.stderr, "Network error; abort."
  sys.exit(-1)

while True:
  try:
    msg = msgs.recv(conn)
    if "data" in msg:
      sys.stdout.write(msg["data"])
    else:
      break
  except msgs.Error:
    print >> sys.stderr, "Network error; data transfer aborted."
    break  # Transfer aborted.

# EOF
