import argparse
import atexit
from connection import Client
import msgs
import os
import sys


parser = argparse.ArgumentParser(
    description = "Downloads its input and writes it to a path on the originator host of this program's caller, echoing it to standard output as well."
    )
parser.add_argument(
    "path",
    help = "Path to write the input to."
    )
ns = parser.parse_args(sys.argv[1:])

missing = [v for v in ["RESHD_SERVICE", "RESHD_SESSION"] if v not in os.environ]
if any(missing):
  print >> sys.stderr, "Missing environ variables:", ", ".join(missing)
  sys.exit(-1)

conn = Client(eval(os.environ["RESHD_SERVICE"]))
atexit.register(conn.close)

bufsize = None
xfer = None
try:
  msgs.send(
      conn,
      code = "ltee",
      session = int(os.environ["RESHD_SESSION"]),
      path = ns.path
      )
  resp = msgs.recv(conn)
  if "error" in resp:
    if resp["error"] == "session":
      print >> sys.stderr, "Unknown session ID:", os.environ["RESHD_SESSION"]
    if resp["error"] == "unwritable":
      print >> sys.stderr, "Cannot write to path", resp["path"]
    sys.exit(-2)
  xfer = resp["xfer"]
  bufsize = resp["bufsize"]
except msgs.Error:
  print >> sys.stderr, "Network error; abort."
  sys.exit(-1)

try:
  while True:
    data = os.read(sys.stdin.fileno(), bufsize)
    msg = dict(code = "xfer", xfer = xfer)
    if len(data) > 0:
      msg["data"] = data
      sys.stdout.write(data)
    msgs.send(conn, **msg)
    if len(data) < 1:
      break
except msgs.Error:
  print >> sys.stderr, "Network error; transfer aborted."
  sys.exit(-1)

# EOF
