import json
import struct
import socket


class Error(Exception):
  pass

def send(sock, **msg):
  msg_json = json.dumps(msg)
  msg_str = struct.pack("!H", len(msg_json)) + msg_json
  try:
    while len(msg_str) > 0:
      n = sock.send(msg_str)
      if n == 0:
        raise Error()
      msg_str = msg_str[n:]
  except socket.error:
    raise Error()

def recv(sock):
  msg_len = struct.unpack("!H", recv_exact(sock, 2))[0]
  return json.loads(recv_exact(sock, msg_len))

def recv_exact(sock, n):
  s = ""
  try:
    while len(s) < n:
      r = sock.recv(n - len(s))
      if len(r) == 0:
        raise Error()
      s += r
  except socket.error:
    raise Error()
  return s


# EOF
