from connection import Client
import msgs
import os
from select import select
import sys


def file_unreadable(path):
  try:
    with open(path):
      return False
  except IOError:
    return True

class Upload(object):

  def __init__(self, xfer, paths, conn, bufsize):
    super(Upload, self).__init__()
    self.xfer = xfer
    self.paths = paths
    self.current = None
    self.conn = conn
    self.bufsize = bufsize
    msgs.send(conn, code = "xfer", xfer = xfer, ok = True)

  def current_file(self):
    if self.current is None:
      if len(self.paths) > 0:
        self.current = open(self.paths.pop(0))
    return self.current

  def push_data(self, ready):
    if self.current in ready:
      data = self.current.read(self.bufsize)
      if len(data) == 0:
        self.current.close()
        self.current = None
      else:
        msgs.send(
            self.conn,
            code = "xfer",
            xfer = self.xfer,
            data = data
            )

  def close(self):
    msgs.send(
        self.conn,
        code = "xfer",
        xfer = self.xfer
        )


class Download(object):

  def __init__(self, xfer, local, conn):
    super(Download, self).__init__()
    self.xfer = xfer
    self.local = local
    self.conn = conn

  def is_closed(self):
    return self.local.closed

  def process(self, msg):
    if "data" in msg:
      self.local.write(msg["data"])
    else:
      self.local.close()


if len(sys.argv) < 3:
  print "Usage: {} <addr> <port> [command...]".format(__file__)
  sys.exit(-1)
addr = sys.argv[1]
port = int(sys.argv[2])
command = []
if len(sys.argv) > 3:
  command = sys.argv[3:]

upload_table = {}
download_table = {}
conn = Client((addr, port))
try:
  msgs.send(
      conn,
      code = "command",
      command = " ".join(command) if len(command) > 0 else None,
      bufsize = 1024
      )
  exit_code = None
  input_open = True
  while exit_code is None or len(download_table) > 0:
    ready, _, _ = select(
        [conn] + ([sys.stdin] if input_open else []) + [
          f
          for f in [u.current_file() for u in upload_table.values()]
          if f is not None
          ],
        [],
        []
        )
    if sys.stdin in ready:
      line = os.read(sys.stdin.fileno(), 1024)
      if len(line) < 1:
        input_open = False
        msgs.send(conn, code = "close")
      else:
        msgs.send(conn, code = "data", data = line)
    if conn in ready:
      msg = msgs.recv(conn)
      if msg["code"] == "close":
        sys.stdout.close()
      elif msg["code"] == "terminated":
        exit_code = msg["retcode"]
      elif msg["code"] == "data":
        sys.stdout.write(msg["data"])
      elif msg["code"] == "upload":
        unreadable = [p for p in msg["paths"] if file_unreadable(p)]
        if len(unreadable) > 0:
          msgs.send(
              conn,
              code  = "error",
              error = "unreadable",
              paths = unreadable
              )
        else:
          upload_table[msg["xfer"]] = Upload(
              msg["xfer"],
              msg["paths"],
              conn,
              1024
              )
      elif msg["code"] == "download":
        try:
          local_file = open(msg["path"], "w")
          msgs.send(
              conn,
              code    = "xfer",
              xfer    = msg["xfer"],
              ok      = True,
              bufsize = 1024
              )
          download_table[msg["xfer"]] = Download(
              msg["xfer"],
              local_file,
              conn
              )
        except IOError:
          msgs.send(
              conn,
              code  = "error",
              error = "unwritable",
              path  = msg["path"]
              )
      elif msg["code"] == "xfer":
        if msg["xfer"] in download_table:
          download_table[msg["xfer"]].process(msg)
    for u in upload_table.itervalues():
      u.push_data(ready)
    for x, u in list(upload_table.iteritems()):
      if u.current_file() is None:
        u.close()
        del upload_table[x]
    for x, d in list(download_table.iteritems()):
      if d.is_closed():
        del download_table[x]
  conn.close()
  sys.exit(exit_code)
except msgs.Error:
  print "Network error; abort."
  conn.close()
  sys.exit(-1)


# EOF
