from argparse import ArgumentParser
from test2 import xxd, run
from observe import ObservePackets
import random
import threading


class TestBreakMultiple(ObservePackets):

    def __init__(self, ns, ctrl_file):
        super(TestBreakMultiple, self).__init__(ns, ctrl_file = ctrl_file)
        self.break_ready = threading.Event()
        self.time_break = []
        self.set_up_break(1.0)

    def set_up_break(self, delay = 0.0):
        time_break = random.expovariate(0.5)
        if time_break < 0.1:
            time_break = 0.1
        elif time_break > 5.0:
            time_break = 5.0
        self.time_break.append(time_break)
        self.break_ready.clear()
        self.break_timer = threading.Timer(delay + time_break, self.break_now)
        self.break_timer.start()

    def break_now(self):
        self.break_ready.set()

    def print_mean_break_time(self, template):
        print template.format(
                "Mean time to break: {:.1f} s ({} breaks yet)".format(
                    sum(self.time_break) / float(len(self.time_break)),
                    len(self.time_break) - 1
                    )
                )

    def handle_transfer(self, addr, src, dest, client_to_server):
        frame = self.catch_frame(src, client_to_server)
        if frame is None:
            self.print_mean_break_time("**** CONNECTION OVER -- {} ****")
            self.break_timer.cancel()
            return True
        if self.break_ready.wait(0.0):
            self.print_mean_break_time("**** BREAK CONNECTION -- {} ****")
            self.set_up_break()
            return True
        self.send_all(dest, frame)


if __name__ == '__main__':
    parser = ArgumentParser(
            description = "Multiple connection breaks."
            )
    run(TestBreakMultiple, parser)


# EOF
