from argparse import ArgumentParser
from test2 import xxd, run
from observe import ObservePackets
import random
import threading
import time


class TestClientMultiple(ObservePackets):

    def __init__(self, ns, ctrl_file):
        super(TestClientMultiple, self).__init__(ns, ctrl_file = ctrl_file)
        self.conn = dict()
        self.free_spot = [0, 1, 2]
        self.break_conn = None
        self.break_ready = threading.Event()
        self.time_base = time.time()
        self.break_which = []
        self.schedule_break(4.0, [0])
        self.schedule_break(7.0, [1])
        self.schedule_break(10.0, [2])
        self.schedule_break(13.0, [0,2])
        self.schedule_break(17.0, [0,1,2])

    def schedule_break(self, moment, which):
        threading.Timer(moment, self.break_now, args = (which,)).start()

    def break_now(self, which):
        self.break_which = which
        self.break_ready.set()

    def handle_transfer(self, addr, src, dest, client_to_server):
        key = src if client_to_server else dest
        if key not in self.conn:
            self.conn[key] = self.free_spot.pop(0)
            print "**** GOT CONNECTION: {} ****".format(self.conn[key])
        conn_index = self.conn[key]
        frame = self.catch_frame(src, client_to_server)
        if frame is None:
            print "**** CONNECTION OVER: {} ****".format(conn_index)
            return True
        if self.break_ready.wait(0.0):
            if conn_index in self.break_which:
                print "****"
                print "**** BREAK CONNECTION AT {:.1f}: {} ****".format(
                        time.time() - self.time_base,
                        conn_index
                        )
                print "****"
                self.free_spot.append(conn_index)
                del self.conn[key]
                self.break_which.remove(conn_index)
                if len(self.break_which) == 0:
                    self.break_ready.clear()
                return True
        self.send_all(dest, frame)
        return False


if __name__ == '__main__':
    parser = ArgumentParser(
            description = "Multiple clients with scheduled connection breaks."
            )
    run(TestClientMultiple, parser)


# EOF
