from argparse import ArgumentParser
from test2 import xxd, run
from observe import ObservePackets
import random
import threading


class TestDelayToRecovery(ObservePackets):

    def __init__(self, ns, ctrl_file):
        super(TestDelayToRecovery, self).__init__(ns, ctrl_file = ctrl_file)
        self.break_ready = threading.Event()
        self.set_up_timer(self.break_now, 1.0)

    def set_up_timer(self, meth, delay = 0.0):
        self.break_ready.clear()
        self.timer = threading.Timer(delay + 5.0, meth)
        self.timer.start()

    def break_now(self):
        self.close_server()
        self.set_up_timer(self.recover_now)
        self.break_ready.set()

    def recover_now(self):
        print "**** ALLOW RECONNECTION ****"
        self.set_up_server()
        self.set_up_timer(self.break_now)

    def handle_transfer(self, addr, src, dest, client_to_server):
        frame = self.catch_frame(src, client_to_server)
        if frame is None:
            print "**** CONNECTION OVER ****"
            self.timer.cancel()
            return True
        if self.break_ready.wait(0.0):
            print "**** BREAK AND DELAY RECONNECTION ****"
            return True
        self.send_all(dest, frame)
        return False


if __name__ == '__main__':
    parser = ArgumentParser(
            description = "Multiple connection breaks."
            )
    run(TestDelayToRecovery, parser)


# EOF
