import math
import os.path
import subprocess
import socket
import sys
sys.path.append(
        os.path.abspath(os.path.join(os.path.dirname(__file__), "..", "mitm"))
        )
import mitm


LEN_HEADER = 16
LEN_FOOTER = 12

MEAN_NB_ALTER = 5

GARBAGE_CHARS = list(
        "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz+/012345678=-<>"
        )


def xxd(data, group):
    p = subprocess.Popen(["xxd", "-g", str(group)], stdin = subprocess.PIPE)
    p.communicate(data)
    p.wait()

def get_remote_ctrl(port):
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM, 0)
    sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    sock.bind(("localhost", port))
    sock.listen(5)
    rc, _ = sock.accept()
    sock.close()
    return rc

def run(mitm_klass, arg_parser, port = 9999):
    return mitm.run(mitm_klass, arg_parser, get_remote_ctrl(9999))

def poisson_variate(rnd, mean):
    a = math.exp(-float(mean))
    b = 1.0
    i = 0
    while True:
        u = rnd.random()
        b *= u
        if b < a:
            return i
        i += 1

def nb_bytes_alter_variate(rnd):
    return poisson_variate(rnd, MEAN_NB_ALTER)

def frame_has_distinct_body(frame):
    return len(frame) > LEN_HEADER + LEN_FOOTER

def frame_split(frame):
    header = frame[0:LEN_HEADER]
    footer = frame[-LEN_HEADER:]
    len_body = len(frame) - LEN_HEADER - LEN_FOOTER
    if len_body < 0:
        raise RuntimeError("Frame does not have a distinct body.")
    body = frame[LEN_HEADER : LEN_HEADER + len_body]
    return header, body, footer

def random_char(rnd):
    return chr(rnd.randint(0, 255))

def garbage_str(rnd, glen):
    return "".join(rnd.choice(GARBAGE_CHARS) for _ in xrange(0, glen))

def random_body_index(rnd, frame):
    if len(frame) <= 2*LEN_HEADER + LEN_FOOTER:
        return len(frame) / 2
    return rnd.randint(LEN_HEADER, len(frame) - LEN_HEADER - LEN_FOOTER)


# EOF
