import os
import Queue
import random
import re
import sys
from test import TestCase, run_tests, print_long_output, FILE
import threading
import time


class TestInteractiveShell(TestCase):

    def test(self):
        c = self.clients[0]
        self.cmd(c, "echo 'Hello world!'\n", "Hello world!\n")
        self.cmd(
            c,
            "pwd\n",
            os.path.abspath(self.code_root_join("server")) + "\n"
            )
        with open(self.remote_path("reshd")) as reshd_file:
            self.cmd(c, "cat reshd\n", reshd_file.read())
        c.stdin.close()
        print "Expect return code: 0"
        print "Obtain return code:", self.poll_death(c, 5.0)


class TestPythonInteractive(TestCase):

    def client_list(self):
        return [("python -ui", sys.stdout)]

    def test(self):
        c = self.clients[0]
        time.sleep(0.5)
        print "[Should have got the Python banner by now, with >>> prompt.]"
        self.cmd(c, "print 'Hello world!'\n", "Hello world!\n")
        self.cmd(c, "[n**2 for n in xrange(0,4)]\n", "[0, 1, 4, 9]\n")


class TestNonInteractive(TestCase):

    def remote_files(self):
        return ["asdf"]

    def set_up(self):
        self.asdf_path = self.remote_path("asdf")
        with open(self.asdf_path, "w") as asdf:
            asdf.write("asdf\n")
        super(TestNonInteractive, self).set_up()

    def client_list(self):
        return [("python -u " + test_prog("test_non_interactive.py"), sys.stdout)]

    def test(self):
        c = self.clients[0]
        time.sleep(1.0)
        print "[Expected output: \"Non-interactive program.\\n\"]"
        print "Expect exit code: 4"
        print "Obtain exit code:", self.poll_death(c, 5.0)
        if os.path.exists(self.asdf_path):
            print "FAILURE: program did not delete file 'asdf' as expected."


class TestAlternatingClients(TestCase):

    def client_list(self):
        return [
                ("python -u {} {}".format(rotate_py(), r), sys.stdout)
                for r in [2, 17]
                ]

    def test(self):
        c1, c2 = self.clients
        self.cmd(c1, "ASDF\n", "CUFH\n")
        self.cmd(c2, "QWERTY\n", "HNVIKP\n")
        self.cmd(c1, "HEYHO\n", "JGAJQ\n")
        self.cmd(c2, "BENOIT\n", "SVEFZK\n")
        self.cmd(c2, "DERNIER\n", "UVIEZVI\n")
        self.cmd(c1, "AQUARIUM\n", "CSWCTKWO\n")


class TestSimultaneousClients(TestCase):

    def rot_list(self):
        return [1, 8, 13, 4, 0, 23, 17, -7]

    def client_list(self):
        return [
                ("python -u {} {}".format(rotate_py(), r), FILE)
                for r in self.rot_list()
                ]

    def test(self):
        q = Queue.Queue()
        thr_list = [
                threading.Thread(
                    target = self.run_client,
                    args   = (q, i, rot, c, random.Random())
                    )
                for i, (c, rot) in enumerate(zip(self.clients, self.rot_list()))
                ]
        for t in thr_list:
            t.start()
        for t in thr_list:
            t.join()
        while not q.empty():
            i, r = q.get()
            self.output_data[i]["expected"] = r

    def run_client(self, q, ii, rot, client, rnd):
        s = random_letters(rnd, 64*1024)
        r = rotate(s, rot)
        q.put((ii, r))
        resp = ""
        i = 0
        while True:
            if i >= len(s):
                break
            chunk = s[i:i+1024]
            client.stdin.write(chunk)
            i += 1024
        time.sleep(2.0)  # Finish chewing all that data.
        client.stdin.close()


class TestLateExitCode(TestCase):

    def client_list(self):
        return [("python -u " + test_prog("late_exit_code.py"), sys.stdout)]

    def test(self):
        c = self.clients[0]
        print "[Should get no output -- wait for exit code for ~20 seconds.]"
        print "Expect exit code:", 37
        print "Obtain exit code:", self.poll_death(c, 20.0)


#class TestServerKillOnConnectionLoss(TestCase):
#
#    def test(self):
#        c = self.clients[0]
#        time.sleep(0.2)
#        children_pid = get_children_pid(self.server.pid)
#        print "Expect the server to have 1 child; it actually has", len(children_pid)
#        c.kill()
#        time.sleep(1.0)
#        print "Expect the server to have remained alive; it is", str(is_alive(self.server.pid))
#        print "Expect the server's child to have died; it is", str(not is_alive(children_pid[0]))
#
#
#class TestClientExitCodeOnConnectionLoss(TestCase):
#
#    def test(self):
#        c = self.clients[0]
#        time.sleep(0.2)
#        self.server.kill()
#        time.sleep(0.2)
#        print "Expect the client's return code to be -1 (or equivalent bin representation):", c.wait()


class TestCaseLocalReshd(TestCase):

    def local_files(self):
        return ["local_reshd"]

    def set_up(self):
        super(TestCaseLocalReshd, self).set_up()
        with open(self.remote_path("reshd")) as reshd_file:
            self.reshd_bytes = reshd_file.read()

    def check_local_reshd(self):
        time.sleep(0.2)
        print_long_output("Expect local_reshd", self.reshd_bytes)
        with open(self.local_path("local_reshd")) as lfile:
            print_long_output("Obtain local_reshd", lfile.read())


class TestLteeDownload(TestCaseLocalReshd):

    def test(self):
        c = self.clients[0]
        self.cmd(c, "./ltee local_reshd <reshd >/dev/null\necho $?\n", "0\n")
        self.check_local_reshd()


class TestLteeRedirect(TestCaseLocalReshd):

    def test(self):
        c = self.clients[0]
        self.cmd(c, "cat reshd | ./ltee local_reshd\n", self.reshd_bytes)
        self.check_local_reshd()


class TestCaseRotateXfer(TestCase):

    def data_size(self):
        return 128*1024

    def set_up(self):
        super(TestCaseRotateXfer, self).set_up()
        for p, _, _ in self.work():
            with open(p, "w") as sfile:
                sfile.write(random_letters(random, self.data_size()))

    def nb_xfers(self):
        return len(self.rot_list())

    def file_list(self, prefix):
        return ["{}{}".format(prefix, n) for n in xrange(0, self.nb_xfers())]

    def work(self):
        return zip(self.src_paths(), self.dest_paths(), self.rot_list())

    def rotate_script(self):
        return "python -u " + rotate_py()

    def wait_for_xfers(self):
        time.sleep(1.0)
        if any(not os.path.exists(p) for _, p, _ in self.work()):
            return False
        while any(os.stat(p).st_size < self.data_size() for _, p, _ in self.work()):
            time.sleep(0.1)
        return True

    def check_xfers(self):
        if not self.wait_for_xfers():
            print "***** FAILURE while waiting for data transfers *****"
            return
        for i, (spath, dpath, rot) in enumerate(self.work()):
            with open(spath) as sfile:
                try:
                    with open(dpath) as dfile:
                        expected = rotate(sfile.read(), rot)
                        print_long_output(
                                "Expected data from {}".format(spath),
                                expected
                                )
                        print_long_output(
                                "Obtained data in {}".format(dpath),
                                dfile.read()
                                )
                except IOError:
                    print "***** FAILURE: unable to open file {} *****".format(dpath)


class TestManyLteesOneConnection(TestCaseRotateXfer):

    def rot_list(self):
        return [2, 17, 7]

    def local_files(self):
        return self.file_list("r")

    def remote_files(self):
        return self.file_list("s")

    def src_paths(self):
        return self.remote_paths()

    def dest_paths(self):
        return self.local_paths()

    def test(self):
        c = self.clients[0]
        c.stdin.write(
                " ".join(
                    "{script} {rot} <{src} | ./ltee {dest} >/dev/null &".format(
                        script = self.rotate_script(),
                        rot    = rot,
                        src    = os.path.basename(src),
                        dest   = os.path.basename(dest)
                        )
                    for src, dest, rot in self.work()
                    ) + "\n"
                )
        self.check_xfers()


class TestManyLteesManyConnections(TestCaseRotateXfer):

    def rot_list(self):
        return [13, 22]

    def client_list(self):
        return [("/bin/bash", FILE)] * self.nb_xfers()

    def local_files(self):
        return self.file_list("r")

    def remote_files(self):
        return self.file_list("s")

    def src_paths(self):
        return self.remote_paths()

    def dest_paths(self):
        return self.local_paths()

    def test(self):
        for i, (c, (src, dest, rot)) in enumerate(zip(self.clients, self.work())):
            c.stdin.write(
                    "{script} {rot} <{src} | ./ltee {dest}\n".format(
                        script = self.rotate_script(),
                        rot    = rot,
                        src    = os.path.basename(src),
                        dest    = os.path.basename(dest)
                        )
                    )
            with open(src) as src_file:
                self.output_data[i]["expected"] = rotate(src_file.read(), rot)
        self.check_xfers()


class TestLteeEndProgBeforeEndXfer(TestCaseRotateXfer):

    def rot_list(self):
        return [13]

    def data_size(self):
        return 1024*1024

    def local_files(self):
        return self.file_list("r")

    def remote_files(self):
        return self.file_list("s")

    def src_paths(self):
        return self.remote_paths()

    def dest_paths(self):
        return self.local_paths()

    def test(self):
        c = self.clients[0]
        src, dest, rot = self.work()[0]
        c.stdin.write(
                "{script} {rot} <{src} | ./ltee {dest} >/dev/null\n".format(
                    script = self.rotate_script(),
                    rot    = rot,
                    src    = os.path.basename(src),
                    dest   = os.path.basename(dest)
                    )
                )
        c.stdin.close()
        c.wait()
        self.check_xfers()


class TestCaseRemoteResh(TestCase):

    def remote_files(self):
        return ["remote_code"]

    def set_up(self):
        super(TestCaseRemoteResh, self).set_up()
        with open(self.local_path("resh")) as resh_file:
            self.code_bytes = resh_file.read()
        with open(self.local_path("reshd")) as reshd_file:
            self.code_bytes += reshd_file.read()

    def check_remote_code(self):
        time.sleep(0.2)
        print_long_output("Expect remote_code", self.code_bytes)
        with open(self.remote_path("remote_code")) as rfile:
            print_long_output("Obtain remote_code", rfile.read())


class TestLcatUpload(TestCaseRemoteResh):

    def test(self):
        c = self.clients[0]
        self.cmd(c, "./lcat resh reshd >remote_code\necho $?\n", "0\n")
        self.check_remote_code()


class TestLcatRedirect(TestCaseRemoteResh):

    def test(self):
        c = self.clients[0]
        self.cmd(c, "./lcat resh reshd | tee remote_code\n", self.code_bytes)
        self.check_remote_code()


class TestManyLcatsOneConnection(TestCaseRotateXfer):

    def rot_list(self):
        return [2, 17, 7]

    def local_files(self):
        return self.file_list("s")

    def remote_files(self):
        return self.file_list("r")

    def src_paths(self):
        return self.local_paths()

    def dest_paths(self):
        return self.remote_paths()

    def test(self):
        c = self.clients[0]
        c.stdin.write(
                " ".join(
                    "./lcat {src} | {script} {rot} >{dest} &".format(
                        script = self.rotate_script(),
                        rot    = rot,
                        src    = os.path.basename(src),
                        dest   = os.path.basename(dest)
                        )
                    for src, dest, rot in self.work()
                    ) + "\n"
                )
        self.check_xfers()


class TestManyLcatsManyConnections(TestCaseRotateXfer):

    def rot_list(self):
        return [13, 22]

    def client_list(self):
        return [("/bin/bash", FILE)] * self.nb_xfers()

    def local_files(self):
        return self.file_list("s")

    def remote_files(self):
        return self.file_list("r")

    def src_paths(self):
        return self.local_paths()

    def dest_paths(self):
        return self.remote_paths()

    def test(self):
        for i, (c, (src, dest, rot)) in enumerate(zip(self.clients, self.work())):
            c.stdin.write(
                    "./lcat {src} | {script} {rot} | tee {dest}\n".format(
                        script = self.rotate_script(),
                        rot    = rot,
                        src    = os.path.basename(src),
                        dest   = os.path.basename(dest)
                        )
                    )
            with open(src) as src_file:
                self.output_data[i]["expected"] = rotate(src_file.read(), rot)
        self.check_xfers()


class TestLcatAndLtee(TestCaseRotateXfer):

    def rot_list(self):
        return [13]

    def local_files(self):
        return ["s0", "s1", "s2", "r0"]

    def src_paths(self):
        return [self.local_path("s0")]

    def dest_paths(self):
        return [self.local_path("r0")]

    def set_up(self):
        super(TestLcatAndLtee, self).set_up()
        with open(self.local_path("s0")) as s0_file:
            s0_bytes = s0_file.read()
            n = random.randint(1024, len(s0_bytes) - 1024)
            with open(self.local_path("s1"), "w") as s1_file:
                s1_file.write(s0_bytes[0:n])
            with open(self.local_path("s2"), "w") as s2_file:
                s2_file.write(s0_bytes[n:])

    def test(self):
        c = self.clients[0]
        src, dest, rot = self.work()[0]
        c.stdin.write(
                "./lcat s1 s2 | {script} {rot} | ./ltee r0 >/dev/null\n".format(
                        script = self.rotate_script(),
                        rot    = rot
                        )
                )
        self.check_xfers()
                    

def random_letters(rnd, n):
    return "".join(chr(rnd.randint(ord('A'), ord('Z'))) for n in xrange(0, n))

def rotate(s, rot):
    return "".join(chr((ord(c) - ord('A') + rot) % 26 + ord('A')) for c in s)

def get_children_pid(parent):
    children_pid = []
    for pid in [pp for pp in os.listdir("/proc") if re.match("[0-9]", pp)]:
        with open(os.path.join("/proc", pid, "stat")) as stat_file:
            data = stat_file.read().split()
            ppid = int(data[3])
            if parent == ppid:
                children_pid.append(int(pid))
    return children_pid

def is_alive(pid):
    return str(pid) in os.listdir("/proc")

def test_prog(name):
    return root_join("test", name)

def rotate_py():
    return root_join("test", "rotate.py")

def root_join(*path):
    return os.path.abspath(
            os.path.join(os.path.dirname(__file__), "..", *path)
            )


if __name__ == '__main__':
    run_tests(sys.argv[1:], locals())


# EOF
